from typing import Optional

from cpl_core.application import StartupExtensionABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.console import Console, ForegroundColorEnum
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_discord.service.discord_collection_abc import DiscordCollectionABC
from cpl_query.extension import List

from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from modules.admin.admin_module import AdminModule
from modules.base.base_module import BaseModule
from modules.boot_log.boot_log_module import BootLogModule
from modules.database.database_module import DatabaseModule
from modules.moderator.moderator_module import ModeratorModule
from modules.permission.permission_module import PermissionModule


class StartupModuleExtension(StartupExtensionABC):

    def __init__(self):
        self._config: Optional[ConfigurationABC] = None
        self._feature_flags: Optional[FeatureFlagsSettings] = None

        self._modules = List(type, [
            AdminModule,
            BaseModule,
            DatabaseModule,
            ModeratorModule,
            PermissionModule,
            BootLogModule  # has to be last!
        ])

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        self._config = config
        self._feature_flags = config.get_configuration(FeatureFlagsSettings)

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        provider = services.build_service_provider()
        dc_collection: DiscordCollectionABC = provider.get_service(DiscordCollectionABC)

        for module_type in self._modules:
            module = module_type(dc_collection)
            if not module.feature_flag(self._feature_flags):
                continue

            Console.set_foreground_color(ForegroundColorEnum.green)
            Console.write_line(f'[{__name__}] Loaded module: {module_type}')
            Console.color_reset()
            module.configure_configuration(self._config, env)
            module.configure_services(services, env)
